//
//  MOIconDownloader.m
//  MOThumbnailTableView
//
//  Created by Jan Christiansen on 10/1/12.
//  Copyright (c) 2012 monoid - Software Development and Consulting - Jan Christiansen.
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//  copyright notice, this list of conditions and the following
//  disclaimer in the documentation and/or other materials provided
//  with the distribution.
//
//  * Neither the name of Monoid - Development and Consulting -
//  Jan Christiansen nor the names of other
//  contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import <UIKit/UIKit.h>

#import "MOIconDownloader.h"


/**
 * Private interface of the class
 */
@interface MOIconDownloader ()


/// @name Setting and Getting Properties

@property(strong, nonatomic) NSMutableData *activeDownload;
@property(strong, nonatomic) NSURLConnection *iconConnection;
@property(strong, nonatomic) NSURL *iconBasisURL;


/// release the current connection
- (void)releaseConnection;


@end


@implementation MOIconDownloader

// timeout of a download
static const NSTimeInterval kTimeoutInterval = 20;

@synthesize delegate = _delegate;
@synthesize activeDownload = _activeDownload;
@synthesize iconConnection = _iconConnection;
@synthesize userInfo = _userInfo;
@synthesize iconBasisURL = _iconBasisURL;


#pragma mark - Initializing

- (id)initWithIconBasisURL:(NSURL *)iconBasisURL {
    
    self = [super init];
    if (self) {
        _iconBasisURL = iconBasisURL;
    }
    return self;
}


#pragma mark - Starting and Stopping downloads

- (void)startDownload {
    
    NSLog(@"Start download of URL: %@", self.iconBasisURL);

    self.activeDownload = [NSMutableData data];
    NSURLRequest *iconRequest = [[NSURLRequest alloc] initWithURL:self.iconBasisURL 
                                                      cachePolicy:NSURLRequestReturnCacheDataElseLoad 
                                                  timeoutInterval:kTimeoutInterval];
    self.iconConnection = [[NSURLConnection alloc] initWithRequest:iconRequest
                                                          delegate:self];
}

- (void)cancelDownload {
    
    [self.iconConnection cancel];
    [self releaseConnection];
}


#pragma mark - NSURLConnectionDelegate Methods

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
    
    [self.activeDownload appendData:data];
}

- (void)connection:(NSURLConnection *)connection 
didReceiveResponse:(NSURLResponse *)response {

    // if a download fails with an error code we invoke 
    // connection:didFailWithError:
    if ([response respondsToSelector:@selector(statusCode)]) {
        
        int statusCode = [((NSHTTPURLResponse *)response) statusCode];
        if (statusCode >= 400) {

            [self.iconConnection cancel];
            NSString *errorDescription = [NSString stringWithFormat:NSLocalizedString(@"ErrorCodeFormat", nil), statusCode];
            NSDictionary *errorInfo = [NSDictionary dictionaryWithObject:errorDescription
                                                                  forKey:NSLocalizedDescriptionKey];
            NSError *statusError = [NSError errorWithDomain:NSLocalizedString(@"ErrorDomain", nil)
                                                       code:statusCode
                                                   userInfo:errorInfo];
            [self connection:connection didFailWithError:statusError];
        }
    }
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
 
    // inform the delegate about the failure
    [self.delegate iconDownloader:self didFailWithError:error];
    [self releaseConnection];
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection {

    // inform the delegate about the successful download
    UIImage *iconImage = [[UIImage alloc] initWithData:self.activeDownload];
    [self.delegate iconDownloader:self didLoadIcon:iconImage];
    [self releaseConnection];
}


#pragma mark - Release

- (void)releaseConnection {
    
    self.activeDownload = nil;
    self.iconConnection = nil;
}


@end
