//
//  MOIconDownloaderManager.m
//  MOThumbnailTableView
//
//  Created by Jan Christiansen on 10/1/12.
//  Copyright (c) 2012 monoid - Software Development and Consulting - Jan Christiansen.
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//  * Redistributions of source code must retain the above copyright
//  notice, this list of conditions and the following disclaimer.
//
//  * Redistributions in binary form must reproduce the above
//  copyright notice, this list of conditions and the following
//  disclaimer in the documentation and/or other materials provided
//  with the distribution.
//
//  * Neither the name of Monoid - Development and Consulting -
//  Jan Christiansen nor the names of other
//  contributors may be used to endorse or promote products derived
//  from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//

#import <UIKit/UIKit.h>

#import "MOIconDownloaderManager.h"
#import "MOIconDownloader.h"


/**
 * Private interface of the class
 */
@interface MOIconDownloaderManager ()


/// @name Setting and Getting Properties

// dictionary of active downloads where the key is the icon file name
@property(strong, nonatomic) NSMutableDictionary *activeDownloads;

// dictionary of arrays of delayed downloads where the key is the icon file name
@property(strong, nonatomic) NSMutableDictionary *delayedDownloads;


#pragma mark - Utility Methods

- (void)startDelayedDownloader:(MOIconDownloader *)iconDownloader;


@end


/**
 * Manages multiple downloads and starts only a single download per icon file
 * name. Caching is done by the caching mechanism of `NSURLConnection`.
 */
@implementation MOIconDownloaderManager


@synthesize delegate = _delegate;
@synthesize iconBasisURL = _iconBasisURL;
@synthesize activeDownloads = _activeDownloads;
@synthesize delayedDownloads = _delayedDownloads;


#pragma mark - Constants

// key used to store the icon file name in the user dictionary of the downloader
static const NSString *kIconFileNameDictionaryKey = @"iconFileName";


#pragma mark - Initialization

- (id)init {

    return [self initWithIconBasisURL:nil];
}

- (id)initWithIconBasisURL:(NSURL *)iconBasisURL {
    
    self = [super init];
    if (self) {
        _iconBasisURL = iconBasisURL;
        _activeDownloads = [NSMutableDictionary dictionary];
        _delayedDownloads = [NSMutableDictionary dictionary];
    }
    return self;
}


#pragma mark - Starting and Stopping Downloads

- (void)startDownloadOfIcon:(NSString *)iconFileName 
               withUserInfo:(NSDictionary *)userInfo {
    
    // construct an icon downloader for the specific icon
    NSURL *iconURL;
    if (self.iconBasisURL) {
        iconURL = [self.iconBasisURL URLByAppendingPathComponent:iconFileName];
    } else {
        iconURL = [NSURL URLWithString:iconFileName];
    }

    MOIconDownloader *iconDownloader = [[MOIconDownloader alloc] initWithIconBasisURL:iconURL];
    iconDownloader.delegate = self;
    NSMutableDictionary *mutableUserInfo = [NSMutableDictionary dictionaryWithDictionary:userInfo]; 
    [mutableUserInfo setObject:iconFileName forKey:kIconFileNameDictionaryKey];
    iconDownloader.userInfo = mutableUserInfo;

    if (![self.activeDownloads objectForKey:iconFileName]) {
        // there is currently no active downlaoder for this iconFileName
        // start download
        [self.activeDownloads setObject:iconDownloader forKey:iconFileName];
        [iconDownloader startDownload];
    } else {
        // a downloader is already running for this iconFileName
        NSMutableArray *delayedDownloadsOfIconFileName = [self.delayedDownloads objectForKey:iconFileName];
        if (!delayedDownloadsOfIconFileName) {
            // there is no array of delayed downloads for this iconFileName
            // add an array
            NSMutableArray *newArray = [NSMutableArray arrayWithObject:iconDownloader];
            [self.delayedDownloads setObject:newArray forKey:iconFileName];
        } else {
            // there are already delayed downloads for this iconFileName
            // add the new delayed download to this array
            [delayedDownloadsOfIconFileName addObject:iconDownloader];
        }
    }
}

- (void)cancelDownloads {
    
    // cancel all active downloads
    for (MOIconDownloader *downloader in self.activeDownloads.allValues) {
        [downloader cancelDownload];
    }
    self.activeDownloads = [NSMutableDictionary dictionary];
    self.delayedDownloads = [NSMutableDictionary dictionary];
}


#pragma mark - IconDownloaderDelegate Methods

- (void)iconDownloader:(MOIconDownloader *)iconDownloader
           didLoadIcon:(UIImage *)icon {
    
    // as one downloader finished, start a delayed downloader for this icon
    [self startDelayedDownloader:iconDownloader];
    
    // inform the delegate that a download finished
    [self.delegate iconDownloader:iconDownloader didLoadIcon:icon];
}

- (void)iconDownloader:(MOIconDownloader *)iconDownloader
      didFailWithError:(NSError *)error {

    // as one downloader failed, start a delayed downloader for this icon
    [self startDelayedDownloader:iconDownloader];
    
    // inform the delegate that a download failed
    [self.delegate iconDownloader:iconDownloader didFailWithError:error];
}


#pragma mark - Utility Method

// used when a download succeeds or fails
- (void)startDelayedDownloader:(MOIconDownloader *)iconDownloader {
    
    NSString *iconFileName = [iconDownloader.userInfo objectForKey:kIconFileNameDictionaryKey];
    NSMutableArray *delayedDownloadsOfIconFileName = [self.delayedDownloads objectForKey:iconFileName];
    
    if (delayedDownloadsOfIconFileName && [delayedDownloadsOfIconFileName count] > 0) {
        // there is a delayed download
        // start it
        MOIconDownloader *newDownloader = [delayedDownloadsOfIconFileName objectAtIndex:0];
        [self.activeDownloads setObject:newDownloader forKey:iconFileName];
        [delayedDownloadsOfIconFileName removeObject:newDownloader];
        [newDownloader startDownload];
    } else {
        // there is no delayed download
        // remove the old download from the array of active downloads
        [self.activeDownloads removeObjectForKey:iconFileName];
    }
}


#pragma mark - Release

- (void)dealloc {

    for (MOIconDownloader *iconDownloader in self.activeDownloads.allValues) {
        [iconDownloader cancelDownload];
    }
}
    
@end
